/*
 * Copyright (c) 2023 Shanghai Panchip Microelectronics Co.,Ltd.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#ifndef _FTM_H
#define _FTM_H

#include <zephyr.h>
#include <bluetooth/bluetooth.h>

typedef enum {
	other                                   = 0x00,
	idle                                    = 0x01,
	warming_Up                              = 0x02,
	low_intensity_interval                  = 0x03,
	high_intensity_interval                 = 0x04,
	recovery_interval                       = 0x05,
	isometric                               = 0x06,
	heart_rate_control                      = 0x07,
	fitness_test                            = 0x08,
	speed_outside_of_control_region_low     = 0x09,
	speed_outside_of_control_region_high    = 0x0A,
	cool_down                               = 0x0B,
	watt_control                            = 0x0C,
	quick_start                             = 0x0D,
	pre_workout                             = 0x0E,
	post_workout                            = 0x0F,
} training_status;

/**
 * treadmill field definition
 */
typedef enum {
	instantaneous_speed                             = 0x0000, /*0 means support instantaneous speed*/
	treadmill_average_speed                         = 0x0002,
	treadmill_total_distance                        = 0x0004,
	inclination_and_ramp_angle_setting              = 0x0008,
	treadmill_elevation_gain                        = 0x0010,
	instantaneous_pace                              = 0x0020,
	average_pace                                    = 0x0040,
	treadmill_expended_energy                       = 0x0080,
	heart_rate                                      = 0x0100,
	treadmill_metabolic_equivalent                  = 0x0200,
	treadmill_elapsed_time                          = 0x0400,
	treadmill_remaining_time                        = 0x0800,
	treadmill_force_on_belt_and_power_output        = 0x1000,
} treadmill_data_field;

/**
 * fitness_machine_feature definition
 */
typedef enum {
	average_speed                   = 0x00000001,
	cadence_supported               = 0x00000002,
	total_distance                  = 0x00000004,
	inclination_supported           = 0x00000008,
	elevation_gain                  = 0x00000010,
	pace_supported                  = 0x00000020,
	step_count                      = 0x00000040,
	resistance_level                = 0x00000080,
	stride_count                    = 0x00000100,
	expended_energy                 = 0x00000200,
	heart_rate_measurement          = 0x00000400,
	metabolic_equivalent            = 0x00000800,
	elapsed_time                    = 0x00001000,
	remaining_time                  = 0x00002000,
	power_measurement               = 0x00004000,
	force_on_belt_and_power_output  = 0x00008000,
	user_data_retention             = 0x00010000,
} fitness_machine_feature;

/**
 * target_setting_feature definition
 *  little endian
 */
typedef enum {
	speed_target_setting                                    = 0x00000001,
	inclination_target_setting                              = 0x00000002,
	resistance_target_setting                               = 0x00000004,
	power_target_setting                                    = 0x00000008,
	heart_rate_target_setting                               = 0x00000010,
	targeted_expended_energy_configuration                  = 0x00000020,
	targeted_step_number_configuration                      = 0x00000040,
	targeted_stride_number_configuration                    = 0x00000080,
	targeted_distance_configuration                         = 0x00000100,
	targeted_training_time_configuration                    = 0x00000200,
	targeted_time_in_two_heart_rate_zones_configuration     = 0x00000400,
	targeted_time_in_three_heart_rate_zones_configuration   = 0x00000800,
	targeted_time_in_five_heart_rate_zones_configuration    = 0x00001000,
	indoor_bike_simulation_parameters                       = 0x00002000,
	wheel_circumference_configuration                       = 0x00004000,
	spin_down_control                                       = 0x00008000,
	targeted_cadence_configuration                          = 0x00010000,
} target_setting_feature;

typedef struct {
	unsigned short flags;
	unsigned short instantaneousSpeed;
	unsigned short averageSpeed;
	unsigned int totalDistance;
	signed short inclination;
	unsigned short positiveElevationGain;
	unsigned short negativeElevationGain;
	unsigned short totalEnergy;
	unsigned short energyPerHour;
	unsigned char energyPerMinute;
	unsigned char heartRate;
	unsigned short elapsedTime;
} treadmill_Data_t;

/**
 * Fitness Machine status definition
 */
typedef enum {
	pause_by_user           = 0x00,
	status_reset            = 0x01,
	stop_by_user            = 0x02,
	stop_by_safety_key      = 0x03,
	start_by_user           = 0x04,
	target_speed_changed    = 0x05,
	target_incline_changed  = 0x06,
	resume_by_user          = 0x16,
	control_permission_lost = 0xFF,
} fitness_machine_status_operation_code;


/**
 * Fitness Machine Control Point response resultCode definition
 */
typedef enum {
	cp_success                      = 0x01,
	cp_op_code_not_supported        = 0x02,
	cp_invalid_parameter            = 0x03,
	cp_operation_failed             = 0x04,
	cp_control_not_permitted        = 0x05,
} fitness_machine_control_point_response_resultCode;

/**
 * Fitness Machine Control Point op code definition
 */
typedef enum {
	cp_request_control              = 0x00,
	cp_reset                        = 0x01,
	cp_set_target_speed             = 0x02,
	cp_set_target_inclination       = 0x03,
	cp_start                        = 0x07,
	cp_stop                         = 0x08,
	cp_response_code                = 0x80,
} fitness_machine_control_point_operation_code;

struct write_ctrl_point_req {
	uint8_t op;
	union {
		uint8_t stop_param;
		uint16_t target_speed;
		int16_t target_inclination;
	};
} __packed;

struct ctrl_point_ind_t {
	uint8_t op;
	uint8_t req_op;
	uint8_t status;
	uint8_t data[];
} __packed;

struct fitness_machine_status_t {
	uint8_t op;
	uint8_t data[];
} __packed;


static void ctrl_point_ind(struct bt_conn *conn, uint8_t req_op, uint8_t status,
			   const void *data, uint16_t data_len);
#endif
